@@ -14,6 +14,9 @@ from aiohttp.web_exceptions import (
     HTTPServiceUnavailable,
 )
 import asyncio
+import nest_asyncio
+
+nest_asyncio.apply()
 from typing import Optional, List, Union, Tuple, Dict, Callable
 from itertools import combinations
 from functools import reduce
@@ -311,13 +314,19 @@ class CompoundResolver:
         agreement_satisfied = False
         # Create input identifier queue
         input_identifiers_queue = queue.Queue()
-        for identifier in input_compound.identifiers:
-            input_identifiers_queue.put(identifier)
+        input_identifiers_list = [
+            (identifier, None) for identifier in input_compound.identifiers
+        ]
+        # for identifier in input_compound.identifiers:
+        #     input_identifiers_queue.put()
 
         # Main loop
-        while not input_identifiers_queue.empty() and not agreement_satisfied:
-            input_identifier = input_identifiers_queue.get()
+        while len(input_identifiers_list) > 0 and not agreement_satisfied:
+            input_identifier, no_go_service = input_identifiers_list[0]
+            input_identifiers_list = input_identifiers_list[1:]
             for service in self._services:
+                if service == no_go_service:
+                    continue
                 for j in range(n_retries):
                     try:
                         output_identifier_types = [
@@ -335,20 +344,27 @@ class CompoundResolver:
                         logger.debug(
                             f"{service} | {input_identifier.value}->{resolved_identifiers}"
                         )
-                        # Standardize identifiers (e.g., SMILES canonicalization)
+
+                        final_resolved_identifiers = []
                         for identifier in resolved_identifiers:
+                            # Add backup identifiers to queue
+                            if identifier.identifier_type in backup_identifier_types:
+                                if (
+                                    (identifier, service) not in input_identifiers_list
+                                    # But not second order backup identifier checks
+                                    and input_identifier.identifier_type
+                                    not in backup_identifier_types
+                                ):
+                                    input_identifiers_list.append((identifier, service))
+                                continue
+
+                            # Standardize identifiers (e.g., SMILES canonicalization)
                             if identifier is not None:
                                 standardize_identifier(identifier)
-                        # Add backup identifiers to queue
-                        for identifier in resolved_identifiers:
-                            if (
-                                identifier.identifier_type in backup_identifier_types
-                                and input_identifier.identifier_type
-                                not in backup_identifier_types
-                            ):
-                                input_identifiers_queue.put(identifier)
-                                resolved_identifiers.remove(identifier)
-                        resolved_identifiers_list.append(resolved_identifiers)
+
+                            final_resolved_identifiers.append(identifier)
+
+                        resolved_identifiers_list.append(final_resolved_identifiers)
                         break
                     except aiohttp_errors as e:  # type: ignore
                         # If server is busy, use exponential backoff
@@ -388,13 +404,16 @@ class CompoundResolver:
                 if agreement_satisfied:
                     break
 
-        # If agreement is 0, then we just want to dedup and return
-        if agreement == 0:
-            resolved_identifiers_list = unique_identifiers(resolved_identifiers_list)
+        # If agreement is 0 or 1, then we just want to dedup and return
+        if agreement == 0 or agreement == 1:
+            resolved_identifiers_list = [
+                unique_identifiers(resolved_identifiers)
+                for resolved_identifiers in resolved_identifiers_list
+            ]
             agreement_satisfied = True
 
         if not agreement_satisfied:
-            error_txt = f"Not sufficient agreement for {input_identifier} (outputs: {resolved_identifiers_list})"
+            error_txt = f"Not sufficient agreement for {input_compound} (outputs: {resolved_identifiers_list})"
             if self.silent:
                 logger.error(error_txt)
                 resolved_identifiers_list = []
@@ -429,7 +448,7 @@ def resolve_identifiers(
     batch_size: int = 100,
     services: Optional[List[Service]] = None,
     silent: Optional[bool] = False,
-) -> List[Tuple[Compound, Union[List[CompoundIdentifier], None]]]:
+) -> List[Tuple[str, Union[List[str], None]]]:
     """Resolve a list of names (or any other identifier) to an identifier type.
 
     Parameters
@@ -457,6 +476,12 @@ def resolve_identifiers(
     silent : bool, optional
         If True, logs errors but does not raise them. Default is False
 
+    Returns
+    -------
+    List of tuples where the first element in each tuple is the input identifier
+    and the second element is a list of resolved identifiers.
+
+
     Example
     -------
 
@@ -474,9 +499,12 @@ def resolve_identifiers(
     This is a convenience function for quickly resolving a list of strings without having
     to create Compound objects.
 
+    Due to the asynchronous nature of the API calls, the function will not
+    return the results in the same order as the input list.
+
     """
     if services is None:
-        services = [PubChem(), CIR()]
+        services = [PubChem(autocomplete=True), CIR()]
     compounds = [
         Compound(
             identifiers=[
@@ -486,10 +514,17 @@ def resolve_identifiers(
         for name in names
     ]
     resolver = CompoundResolver(services=services, silent=silent)
-    return resolver.resolve(
+    results = resolver.resolve(
         input_compounds=compounds,
         output_identifier_type=output_identifier_type,
         backup_identifier_types=backup_identifier_types,
         agreement=agreement,
         batch_size=batch_size,
     )
+    return [
+        (
+            input_compound.identifiers[0].value,
+            [identifier.value for identifier in resolved_identifiers],
+        )
+        for input_compound, resolved_identifiers in results
+    ]
